'use strict';

var querystring = require('querystring');

var executeRequest = require('./executeRequest');

var GOOGLE_AUTH_ENDPOINT = 'https://accounts.google.com/o/oauth2/auth';
var GOOGLE_API_HOST = 'https://www.googleapis.com';
var GOOGLE_API_PATH = '/oauth2/v3/token';
var GOOGLE_API_PATH_NEW = '/oauth2/v4/token';

var PICASA_SCOPE = 'https://picasaweb.google.com/data';
var PICASA_API_FEED_PATH = '/feed/api/user/default';
var PICASA_API_ENTRY_PATH = '/entry/api/user/default';

var FETCH_AS_JSON = 'json';

function Picasa() {
  this.executeRequest = executeRequest;
}

Picasa.prototype.getPhotos = getPhotos;
Picasa.prototype.postPhoto = postPhoto;
Picasa.prototype.deletePhoto = deletePhoto;
Picasa.prototype.getAlbums = getAlbums;
Picasa.prototype.createAlbum = createAlbum;

// Auth utilities
Picasa.prototype.getAuthURL = getAuthURL;
Picasa.prototype.getAccessToken = getAccessToken;
Picasa.prototype.renewAccessToken = renewAccessToken;

function getAlbums(accessToken, options, callback) {
  var accessTokenParams = {
    alt: FETCH_AS_JSON,
    access_token: accessToken
  };

  options = options || {};

  var requestQuery = querystring.stringify(accessTokenParams);

  var requestOptions = {
    url: '' + PICASA_SCOPE + PICASA_API_FEED_PATH + '?' + requestQuery,
    headers: {
      'GData-Version': '2'
    }
  };

  this.executeRequest('get', requestOptions, function (error, body) {
    if (error) return callback(error);

    var albums = body.feed.entry.map(function (entry) {
      return parseEntry(entry, albumSchema);
    }
    //  entry => entry
    );

    callback(null, albums);
  });
}

function deletePhoto(accessToken, albumId, photoId, callback) {
  var requestQuery = querystring.stringify({
    alt: FETCH_AS_JSON,
    access_token: accessToken
  });

  var requestOptions = {
    url: '' + PICASA_SCOPE + PICASA_API_ENTRY_PATH + '/albumid/' + albumId + '/photoid/' + photoId + '?' + requestQuery,
    headers: {
      'If-Match': '*'
    }
  };

  this.executeRequest('del', requestOptions, callback);
}

function createAlbum(accessToken, albumData, callback) {
  var requestQuery = querystring.stringify({
    alt: FETCH_AS_JSON,
    access_token: accessToken
  });

  var albumInfoAtom = '<entry xmlns=\'http://www.w3.org/2005/Atom\'\n                            xmlns:media=\'http://search.yahoo.com/mrss/\'\n                            xmlns:gphoto=\'http://schemas.google.com/photos/2007\'>\n                          <title type=\'text\'>' + albumData.title + '</title>\n                          <summary type=\'text\'>' + albumData.summary + '</summary>\n                          <gphoto:access>private</gphoto:access>\n                          <category scheme=\'http://schemas.google.com/g/2005#kind\'\n                            term=\'http://schemas.google.com/photos/2007#album\'></category>\n                         </entry>';

  var requestOptions = {
    url: '' + PICASA_SCOPE + PICASA_API_FEED_PATH + '?' + requestQuery,
    body: albumInfoAtom,
    headers: { 'Content-Type': 'application/atom+xml' }
  };

  this.executeRequest('post', requestOptions, function (error, body) {
    if (error) return callback(error);

    var album = parseEntry(body.entry, albumSchema);

    callback(error, album);
  });
}

function postPhoto(accessToken, albumId, photoData, callback) {
  var requestQuery = querystring.stringify({
    alt: FETCH_AS_JSON,
    access_token: accessToken
  });

  var photoInfoAtom = '<entry xmlns="http://www.w3.org/2005/Atom">\n                          <title>' + photoData.title + '</title>\n                          <summary>' + photoData.summary + '</summary>\n                          <category scheme="http://schemas.google.com/g/2005#kind" term="http://schemas.google.com/photos/2007#photo"/>\n                        </entry>';

  var requestOptions = {
    url: '' + PICASA_SCOPE + PICASA_API_FEED_PATH + '/albumid/' + albumId + '?' + requestQuery,
    multipart: [{ 'Content-Type': 'application/atom+xml', body: photoInfoAtom }, { 'Content-Type': photoData.contentType, body: photoData.binary }]
  };

  this.executeRequest('post', requestOptions, function (error, body) {
    if (error) return callback(error);

    var photo = parseEntry(body.entry, photoSchema);

    callback(error, photo);
  });
}

function getPhotos(accessToken, options, callback) {
  var accessTokenParams = {
    alt: FETCH_AS_JSON,
    kind: 'photo',
    access_token: accessToken
  };

  options = options || {};

  if (options.maxResults) accessTokenParams['max-results'] = options.maxResults;

  var albumPart = options.albumId ? '/albumid/' + options.albumId : '';

  var requestQuery = querystring.stringify(accessTokenParams);

  var requestOptions = {
    url: '' + PICASA_SCOPE + PICASA_API_FEED_PATH + albumPart + '?' + requestQuery,
    headers: {
      'GData-Version': '2'
    }
  };

  this.executeRequest('get', requestOptions, function (error, body) {
    if (error) return callback(error);

    var photos = body.feed.entry.map(function (entry) {
      return parseEntry(entry, photoSchema);
    });

    callback(null, photos);
  });
}

var albumSchema = {
  'gphoto$id': 'id',
  'gphoto$name': 'name',
  'gphoto$numphotos': 'num_photos',
  'published': 'published',
  'title': 'title',
  'summary': 'summary',
  'gphoto$location': 'location',
  'gphoto$nickname': 'nickname',
  'media$group': 'media'
};

var photoSchema = {
  'gphoto$id': 'id',
  'gphoto$albumid': 'album_id',
  'gphoto$access': 'access',
  'gphoto$width': 'width',
  'gphoto$height': 'height',
  'gphoto$size': 'size',
  'gphoto$checksum': 'checksum',
  'gphoto$timestamp': 'timestamp',
  'gphoto$imageVersion': 'image_version',
  'gphoto$commentingEnabled': 'commenting_enabled',
  'gphoto$commentCount': 'comment_count',
  'content': 'content',
  'title': 'title',
  'summary': 'summary'
};

function parseEntry(entry, schema) {
  var photo = {};

  Object.keys(schema).forEach(function (schemaKey) {
    var key = schema[schemaKey];

    if (key) {
      var value = checkParam(entry[schemaKey]);

      photo[key] = value;
    }
  });

  return photo;
}

function getAuthURL(config) {
  var authenticationParams = {
    access_type: 'offline',
    scope: '' + PICASA_SCOPE,
    response_type: 'code',
    client_id: config.clientId,
    redirect_uri: config.redirectURI
  };

  var authenticationQuery = querystring.stringify(authenticationParams);

  return GOOGLE_AUTH_ENDPOINT + '?' + authenticationQuery;
}

function getAccessToken(config, code, callback) {
  var accessTokenParams = {
    grant_type: 'authorization_code',
    code: code,
    redirect_uri: config.redirectURI,
    client_id: config.clientId,
    client_secret: config.clientSecret
  };

  var requestQuery = querystring.stringify(accessTokenParams);
  var options = {
    url: '' + GOOGLE_API_HOST + GOOGLE_API_PATH + '?' + requestQuery
  };

  this.executeRequest('post', options, function (error, body) {
    if (error) return callback(error);

    callback(null, body.access_token, body.refresh_token);
  });
}

function renewAccessToken(config, refresh_token, callback) {
  var refreshTokenParams = {
    grant_type: 'refresh_token',
    client_id: config.clientId,
    client_secret: config.clientSecret,
    refresh_token: refresh_token
  };

  var requestQuery = querystring.stringify(refreshTokenParams);
  var options = {
    url: '' + GOOGLE_API_HOST + GOOGLE_API_PATH_NEW + '?' + requestQuery
  };

  this.executeRequest('post', options, function (error, body) {
    if (error) return callback(error);

    callback(null, body.access_token);
  });
}

function checkParam(param) {
  if (param === undefined) return '';else if (isValidType(param)) return param;else if (isValidType(param['$t'])) return param['$t'];else return param;
}

function isValidType(value) {
  return typeof value === 'string' || typeof value === 'number';
}

module.exports = Picasa;